'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS boot program:   OS.BIN           |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)|
'' |  Modifications: (c)2012 "Cluso99" (Ray Rodrick)                          |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' This OS boot program sets up Upper HUB Memory according to the hardware,
''   then calls the the version module (_VER.CMD),
''   which then calls the command module (_CMD.CMD)
''  
'' The command module issues the prompt "SD:>" and waits for a command,
''   then executes the command internally, or externally or by calling
''   an external module (_xxxxxxx.CMD) or a binary file (xxxxxxxx.BIN)
''
'' All external command modules will return to _CMD.CMD after execution.
''
'' Some OS modules are derived from: 
''      KyeDOS v3 (c) 2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)
''      Sphinx    (c) 2009-10 by Michael Park and others
'' Some concepts and code is from Sphinx (Michael Park) & Spinix (Dave Hein).
'' 
'' Special thanks to the following authors for their ideas and MIT code:
''   Michael Park, Dave Hein, Kye, James Moxham, Mike Green, Tomas Rokicki,
''   lonesock, hippy, Jeff Ledger, Roger Williams, Michelli Scales, Andy Schenk.
''   My apologies to any others I may have missed.
'' And of course to Chip Gracey for the fabulous Prop chip and the Spin Interpreter.
'' +--------------------------------------------------------------------------+
'' Modifications by Cluso99...
'' RR20120306   000   Initial module _DOS.CMD
'' RR20120323   004   v0.40 "OS.BIN" 
''

CON
  _clkmode = _hw#_clkmode_
  _xinfreq = _hw#_xinfreq_

  _nullCharacter                = 0
  _homeCursorCharacter          = 1
  _backspaceCharacter           = 8
  _horizontalTabCharacter       = 9
  _lineFeedCharacter            = 10
  _carriageReturnCharacter      = 13
  _clearScreenCharacter         = 16
  _quoteCharacter               = 34
  _backspaceKeycode             = 200

OBJ
  _hw   : "_hwdef"              'hardware pin definitions etc
  _hub  : "_hubdef2"            'fixed hub definitions ($7FFF... down)

  fat   : "SD-MMC_FATEngine_006.spin"
  taf   : "SD-MMC_FATEngine_006.spin"
  str   : "ASCII0_STREngine.spin"
  std   : "_PCSIO.spin"                 ' pc i/o driver 
  stdin : "_STDIN.spin"                 ' use std input driver
  stdout: "_STDOUT.spin"                ' use std output driver

PUB Start | _SDpins, _SIOpins, _SIObaud, cog, z,z0,z1,z2,z3,z4 

{------------------------------------------------------------------------------}
'Determine what hardware we are running on
' preset for defined configuration in _hwdef
  _SIOpins := _hw#_SIOpins
  _SIObaud := _hw#_SIObaud
  _SDpins  := _hw#_SDpins                               
' now check for RamBlade/RamBlade3/TriBlade#2
  z := ina                                              'read the input pins (to check pullups)
  if     (z & _hw#_tb2PU) == _HW#_tb2PU                 'TriBlade#2 ?
    _SDpins  := _hw#_tb2
  elseif (z & _hw#_rb1PU) == _hw#_rb1PU                 'RamBlade ?
    _SIOpins := _hw#_sioRB1                             ' serial pins differ on RB1
    _SDpins := _hw#_rb1
  elseif (z & _hw#_rb3PU) == _hw#_rb3PU                 'RamBlade3 ?
    _SDpins := _hw#_rb3
{------------------------------------------------------------------------------}
'Set parameters in hub for other modules
  long[_hub#_HubFree]     := _hub#_SetHubFree           ' set top of hub free
  byte[_hub#_OS_COGS]     := 0                          ' set resident cogs (not to be stopped)
  byte[_hub#_OS_CLKMODE]  := byte[$0004]                ' save the current clockmode
  byte[_hub#_OS_BITS]     := !_hub#_OS_LFmask           ' <lf> off
  byte[_hub#_OS_UNDEF]    := 0                          ' undefined
  long[_hub#_OS_CLKFREQ]  := long[$0000]                ' save the current clockfreq
  long[_hub#_SDpins]      := _SDpins & $1F1F1F1F        
  long[_hub#_SIOpins]     := (cog-1) << 24 | _SIOpins
  long[_hub#_SIObaud]     := _SIObaud                              
  long[_hub#_Hardware]    := 1 << 16 | _SDpins >> 29    ' $0001_000x = cluso's hardware
  str.copy(string("ver"),_hub#_StringBuffer)           
{------------------------------------------------------------------------------}
'Start serial
'                 (inRENDEZVOUS, outRENDEZVOUS, serPins,  baud    )
  cog := std.start(_hub#_StdIn,  _hub#_StdOut,  _SIOpins, _SIObaud) ' start the driver (returns cog+1)
'             (inRENDEZVOUS )
  stdin.start (_hub#_StdIn  )                           ' start the stub
'             (outRENDEZVOUS)
  stdout.start(_hub#_StdOut )                           ' start the stub
  byte[_hub#_OS_COGS]     := (1 << (cog -1))            ' set resident cog
{------------------------------------------------------------------------------}
'Start SD driver
  z  :=  long[_hub#_sdPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
  z3 := (z >> 24) & $1F
'                          (DO, CLK, DI, CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1)
  ifnot( fat.FATEngineStart(z0, z1,  z2, z3, -1,     -1,     -1,       -1,       -1))
    reboot
{------------------------------------------------------------------------------}
'Access the SD card
  PrintStringCR(string("Checking SD card..."))
  fat.mountPartition(0)                                 ' mount the sd card
  ifnot(fat.partitionMounted)
    PrintString(string("Card Not Mounted "))   
  PrintDosVer                                           ' program message
{------------------------------------------------------------------------------}
'Call the "_VER.CMD"
  waitcnt(clkfreq / 10 + cnt)                           ' delay 
  \str.copy(string("ver"),_hub#_stringBuffer)          
  \fat.bootPartition(string("_VER.CMD"))
{------------------------------------------------------------------------------}
'Only here on error
  PrintStringCR(string("=== error ==="))
  waitcnt(clkfreq * 2 + cnt)                            ' delay 
  long[_hub#_HubFree]  := $8000                         ' clear the hubflag!!
  long[_hub#_OS_COGS]  := 0                             ' clear the os cogs!! 
  reboot
{------------------------------------------------------------------------------}

PRI PrintDosVer
    PrintChar(_clearScreenCharacter)                    ' clear screen
    PrintStringCR(string("=== Propeller SD card Operating System V0.04 ==="))
    crlf

PUB PrintStringCR(pstring)      ' sends string to output devices (vga/serial) with CRLF
    PrintString(pstring)                            
    crlf

PUB PrintString(pstring)        ' sends string to output devices (vga/serial) with no CRLF
    repeat strsize(pstring)
      PrintChar(byte[pstring++])
     
PUB PrintChar(c)                ' sends character to output devices (vga/serial)
    if c <> _lineFeedCharacter
      stdout.out(c)                                     ' send char  to output device
    elseif byte[_hub#_OS_BITS] & _hub#_OS_LFmask       
      stdout.out(c)                                     ' send <lf>  to output device

PUB CRLF
    PrintChar(_carriageReturnCharacter)                 ' cr
    PrintChar(_lineFeedCharacter)                       ' lf (optional)

dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}