'' +--------------------------------------------------------------------------+
'' | ......OS   An Operating System for the Propeller Chip                    |
'' +--------------------------------------------------------------------------+
'' |                     HUB DEFINITIONS                                      |
'' +--------------------------------------------------------------------------+
'' |  License            MIT License                                          |
'' +--------------------------------------------------------------------------+
'' RR20120304   Proposal of hub layout
'' RR20120306   add _SDPins, _SIOPins, _SIOBaud
''              add _toggleLF, _hardware
'' RR20120310   add _restartCogs  (could combine as b24-31 in _HubFree???) 
'' RR20120321
'' RR20120323   revised hub layout

'' WARNING: If anything moves, all modules will require re-compiling !!!

CON

' OS Hub Definitions...

  _HubFree      = $7FFC       '    4       ' \ stores total hub available (typ $7800)
                                           ' |  (hub is allocated to the OS above this value)
                                           ' /  (eg $7000 means $0000-$6FFF is available)

'                                        ' \ os runtime configuration parameters
  _OS_COGS      = $7FFB     '      1     ' | b24-31: 1= stay resident cogs (don't stop on reboot)
  _OS_CLKMODE   = $7FFA     '      1     ' | b23-16: saved clkmode hub byte $0004
  _OS_BITS      = $7FF9     '      1     ' | b08-15: runtime config bits
    _OS_LFmask    = $80                  ' |     b15:  1= <lf> ON; 0= strip <lf>
  _OS_UNDEF     = $7FF8     '      1     ' / b00-07: undefined

  _OS_CLKFREQ   = $7FF4       '    4       '   saved clkfreq hub long $0000  

  _SDpins       = $7FF0       '    4       ' \ sd pins packed 4 bytes
                              '            ' / Byte 3=/CS, 2=DI, 1=CLK, 0=DO
  _SIOpins      = $7FEC       '    4       ' \ serial pins and mode settings (and cog#)
                              '            ' / Byte 3=cog, 2=mode, 1=SO, 0=SI
  _SIObaud      = $7FE8       '    4       ' serial baud (speed typ 115,200)

  _StdIn        = $7FE4       '    4       ' standard input  rendezvous
  _StdOut       = $7FE0       '    4       ' standard output rendezvous
  _AuxIn        = $7FDC       '    4       ' auxilary input  rendezvous
  _AuxOut       = $7FD8       '    4       ' auxilary output rendezvous
  _User1        = $7FD4       '    4       ' user 1          rendezvous
'  _User2        = $7FD0       '    4       ' user 2          rendezvous
  _Hardware     = $7FD0       '    4       ' \ hardware: hi-word=company, lo-word=config
                                           ' |            $0001 = Cluso99  $0001 = RamBlade 1           
                                           ' |            $0001 = Cluso99  $0002 = TriBlade#2           
                                           ' /            $0001 = Cluso99  $0003 = RamBlade3           

  _OSreserved   = $7F80       '   80       ' undefined (maybe serial in and serial out buffers?)

  _StringBuffer = $7F00       '  128       ' general purpose string buffer

  _CogTables    = $7E00       '  256       ' \ 8*32B cog usage tables
                                           ' | TBD
                                           ' |   Current thinking is...
                                           ' |     1 byte for "type"
                                           ' |     1 long for sector address to retrieve filename
                                           ' |   However, could also be used as 2*16byte buffers
                                           ' /     with the head+tail(s) kept in _StdIn & _StdOut
  
  _SDbuffer     = $7C00       '  512       ' SD card i/o buffer                 
  _SpinVector   = $7800       ' 1024       ' vector table for Cluso's faster spin interpreter

  _SetHubFree   = $7800                    ' $7800-$7FFF currently reserved by the OS

  
PUB dummy